# -*- coding: utf-8 -*-

"""Main module."""

from abc import ABC, abstractmethod
from pathlib import Path
from typing import Optional

from ghost_writer.utils.logging import setup_logger


class ScriptGenerator(ABC):
    def __init__(self, file_name: str) -> None:

        self._file_name: str = file_name
        self._output: str = ""
        self._build_script()

    @abstractmethod
    def _build_script(self) -> None:
        pass

    def _add_line(self, line: str, indent_level: int = 0) -> None:

        for i in range(indent_level):

            self._output += " " * 4

        # add the line

        self._output += line

        # close the line
        self._end_line()

    @property
    def file_name(self) -> str:
        return self._file_name

    def _end_line(self):

        self._output += "\n"

    def write(self, directory: str = ".") -> None:

        out_file: Path = Path(directory) / self._file_name

        with out_file.open("w") as f:

            f.write(self._output)



from ghost_writer import ScriptGenerator


class NewGenerator(ScriptGenerator):
    def __init__(self, file_name: str) -> None:
        super().__init__(file_name)

    def _build_script(self) -> None:
        self._add_line('import os')
        self._end_line()
        self._end_line()
        self._add_line('for i in range(10):')
        self._add_line('for j in range(20):', indent_level=1)
        self._add_line('pass', indent_level=2)



# -*- coding: utf-8 -*-

"""Top-level package for ghost_writer."""

__author__ = """J. Michael Burgess"""
__email__ = 'jburgess@mpe.mpg.de'


from .ghost_writer import ScriptGenerator
from .scriptify import scriptify_python

from .utils.logging import ghost_writer_config

from ._version import get_versions

__version__ = get_versions()['version']
del get_versions



from pathlib import Path
from typing import List

from .ghost_writer import ScriptGenerator
from .utils.logging import setup_logger

log = setup_logger(__name__)


class PythonMetaGenerator(ScriptGenerator):
    def __init__(
        self, file_name: str, lines: List[str], indents: List[int]
    ) -> None:

        """

        This will generate a generator from an
        existing python script

        :param file_name:
        :type file_name: str
        :param lines:
        :type lines: List[str]
        :param indents:
        :type indents: List[int]
        :returns:

        """
        self._lines: List[str] = lines
        self._indents: List[int] = indents

        super().__init__(file_name)

    def _build_script(self) -> None:

        self._add_line("from ghost_writer import ScriptGenerator")
        self._end_line()
        self._end_line()

        self._add_line("class NewGenerator(ScriptGenerator):")
        self._add_line(
            "def __init__(self, file_name: str) -> None:", indent_level=1
        )
        self._add_line("super().__init__(file_name)", indent_level=2)
        self._end_line()

        self._add_line("def _build_script(self) -> None:", indent_level=1)

        for line, indent in zip(self._lines, self._indents):

            if line == "":

                self._add_line("self._end_line()", indent_level=2)

            else:

                if indent == 0:

                    arg = ""

                else:

                    arg = f", indent_level={indent}"

                self._add_line(
                    f"self._add_line('{line}'{arg})",
                    indent_level=2,
                )


def scriptify_python(file_name: str) -> None:

    """
    Convert a python script into a generator that
    can be used to parameterize the original script.

    A file will created named generated_<filename>.py

    :param file_name:
    :type file_name: str
    :returns:

    """
    gen_lines: List[str] = []
    indents: List[int] = []
    with Path(file_name).open("r") as f:

        script_lines: List[str] = f.readlines()

    for line in script_lines:

        tmp = []

        indent_factor = 1

        if line.startswith(" "):

            indent_factor = 4

            for char in line:

                if char == " ":

                    tmp.append(char)

                else:

                    break

        elif line.startswith("\t"):

            for char in line:

                if char == "\t":

                    tmp.append(char)

                else:

                    break

        num_indents = len(tmp) // indent_factor

        gen_lines.append(line[num_indents:].strip())

        indents.append(num_indents)

    # directory = Path(file_name).parent

    name = Path(file_name).name

    out_file = f"generated_{name}"

    script_gen = PythonMetaGenerator(out_file, gen_lines, indents)

    script_gen.write()

    log.info(f"{out_file}")



import logging
import logging.handlers as handlers
import sys

from rich.console import Console
from rich.logging import RichHandler
from rich.theme import Theme

from .configuration import ghost_writer_config

from .package_data import (
    get_path_of_data_file,
    get_path_of_log_dir,
    get_path_of_log_file,
)

_log_file_names = ["usr.log", "dev.log"]


class LogFilter(object):
    def __init__(self, level):
        self.__level = level

    def filter(self, logRecord):
        return logRecord.levelno != self.__level


# now create the developer handler that rotates every day and keeps
# 10 days worth of backup
ghost_writer_dev_log_handler = handlers.TimedRotatingFileHandler(
    get_path_of_log_file("dev.log"), when="D", interval=1, backupCount=10
)

# lots of info written out

_dev_formatter = logging.Formatter(
    "%(asctime)s | %(name)s | %(levelname)s| %(funcName)s | %(lineno)d | %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)

ghost_writer_dev_log_handler.setFormatter(_dev_formatter)
ghost_writer_dev_log_handler.setLevel(logging.DEBUG)
# now set up the usr log which will save the info

ghost_writer_usr_log_handler = handlers.TimedRotatingFileHandler(
    get_path_of_log_file("usr.log"), when="D", interval=1, backupCount=10
)

ghost_writer_usr_log_handler.setLevel(logging.INFO)

# lots of info written out
_usr_formatter = logging.Formatter(
    "%(asctime)s | %(levelname)s | %(message)s", datefmt="%Y-%m-%d %H:%M:%S"
)

ghost_writer_usr_log_handler.setFormatter(_usr_formatter)

mytheme = Theme().read(get_path_of_data_file("log_theme.ini"))
console = Console(theme=mytheme)

_console_formatter = logging.Formatter(
    ' %(message)s',
    datefmt="%H:%M:%S",
)


ghost_writer_console_log_handler = RichHandler(
    level=ghost_writer_config.logging.level,
    rich_tracebacks=True,
    markup=True,
    console=console,
)
ghost_writer_console_log_handler.setFormatter(_console_formatter)


warning_filter = LogFilter(logging.WARNING)


def silence_warnings():
    """
    supress warning messages in console and file usr logs
    """

    ghost_writer_usr_log_handler.addFilter(warning_filter)
    ghost_writer_console_log_handler.addFilter(warning_filter)


def activate_warnings():
    """
    supress warning messages in console and file usr logs
    """

    ghost_writer_usr_log_handler.removeFilter(warning_filter)
    ghost_writer_console_log_handler.removeFilter(warning_filter)


def update_logging_level(level):

    ghost_writer_console_log_handler.setLevel(level)


def setup_logger(name):

    # A logger with name name will be created
    # and then add it to the print stream
    log = logging.getLogger(name)

    # this must be set to allow debug messages through
    log.setLevel(logging.DEBUG)

    # add the handlers

    log.addHandler(ghost_writer_dev_log_handler)

    log.addHandler(ghost_writer_console_log_handler)

    log.addHandler(ghost_writer_usr_log_handler)

    # we do not want to duplicate teh messages in the parents
    log.propagate = False

    return log



from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional

from omegaconf import OmegaConf
from omegaconf.dictconfig import DictConfig
from rich.tree import Tree

# Path to configuration

_config_path = Path("~/.config/ghost_writer/").expanduser()

_config_name = Path("ghost_writer_config.yml")

_config_file = _config_path / _config_name

# Define structure of configuration with dataclasses


@dataclass
class Logging:

    on: bool = True
    level: str = "WARNING"


@dataclass
class ghost_writerConfig:

    logging: Logging = Logging()


# Read the default config
ghost_writer_config: ghost_writerConfig = OmegaConf.structured(
    ghost_writerConfig
)

# Merge with local config if it exists
if _config_file.is_file():

    _local_config = OmegaConf.load(_config_file)

    ghost_writer_config: ghost_writerConfig = OmegaConf.merge(
        ghost_writer_config, _local_config
    )

# Write defaults if not
else:

    # Make directory if needed
    _config_path.mkdir(parents=True, exist_ok=True)

    with _config_file.open("w") as f:

        OmegaConf.save(config=ghost_writer_config, f=f.name)


def recurse_dict(d, tree) -> None:

    for k, v in d.items():

        if (type(v) == dict) or isinstance(v, DictConfig):

            branch = tree.add(
                k, guide_style="bold medium_orchid", style="bold medium_orchid"
            )

            recurse_dict(v, branch)

        else:

            tree.add(
                f"{k}: [blink cornflower_blue]{v}",
                guide_style="medium_spring_green",
                style="medium_spring_green",
            )

    return


def show_configuration() -> None:

    tree = Tree(
        "config", guide_style="bold medium_orchid", style="bold medium_orchid"
    )

    recurse_dict(ghost_writer_config, tree)

    return tree



import click

from ..scriptify import scriptify_python


@click.command()
@click.argument('file_name')
def scriptify_py(file_name: str) -> None:
    """
    Convert a python script into a generator that
    can be used to parameterize the original script.

    A file will created named generated_<filename>.py

    :param file_name:
    :type file_name: str
    :returns:

    """
    scriptify_python(file_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-


import versioneer
from setuptools import setup


setup(
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    entry_points={
        'console_scripts': [
            'scriptify_py = ghost_writer.scripts.cmd_line:scriptify_py',
        ]
    },
)
